package com.novell.ldap.controls;

import com.novell.ldap.*;
import com.novell.ldap.asn1.*;
import com.novell.ldap.client.Debug;

/**
 *  LDAPPagedResultsControl is a Server Control to specify how search results
 *  are to be returned in pages of specified size by the server. If the control
 *  was marked "critical", the whole search operation will fail if the paged results
 *  control is not supported.
 *
 *  This control is specified in RFC2696
 */
public class LDAPPagedResultsControl extends LDAPControl {

    /**
     * The requestOID of the paged results control
     */
    private static String requestOID = "1.2.840.113556.1.4.319";

    /**
     * The responseOID of the paged results control
     */
    private static String responseOID = "1.2.840.113556.1.4.319";

    /**
     * Empty cookie
     */
    private static final byte EMPTY_COOKIE[] = new byte[0];

    private int _pageSize;
    private byte _cookie[];

    /*
     * This is where we register the control responses
     */
    static
    {
		/*
         * Register the Server Paged Results Control class which is returned by the
		 * server in response to a Paged Results Request
		 */
        try {
            LDAPControl.register(responseOID,
                    Class.forName("com.novell.ldap.controls.LDAPPagedResultsResponse"));
            if( Debug.LDAP_DEBUG) {
                Debug.trace( Debug.controls,
                  "Registered Paged Results Control Response Class");
            }
        } catch (ClassNotFoundException e) {
            if( Debug.LDAP_DEBUG) {
                Debug.trace( Debug.controls,
                  "Could not register Paged Results Control Response - Class not found");
            }
        }
    }

    /**
     * Constructs a paged results control with requested page size and cookie
     *
     *  @param pageSize  Requested page size.
     *
     *  @param cookie    Paged Results cookie.
     *
     *  @param critical	True if the search operation is to fail if the
     *					server does not support this control.
     */
    public LDAPPagedResultsControl(int pageSize, byte cookie[], boolean critical)
    {
        super(requestOID, critical, null);
        _pageSize = pageSize;
        _cookie = cookie;

        setEncodedValue();
    }

    private void setEncodedValue()
    {

        /* Create a new ASN1Sequence object */
        ASN1Sequence m_prRequest = new ASN1Sequence();

        m_prRequest.add(new ASN1Integer(_pageSize));
        m_prRequest.add(new ASN1OctetString((_cookie == null) ? EMPTY_COOKIE : _cookie));

        /* Set the request data field in the in the parent LDAPControl to
         * the ASN.1 encoded value of this control.  This encoding will be
         * appended to the search request when the control is sent.
         */
        setValue (m_prRequest.getEncoding(new LBEREncoder()));

    }

    /**
     * Constructs a paged results control with requested page size
     *
     *  @param pageSize  Requested page size.
     *
     *  @param critical	True if the search operation is to fail if the
     *					server does not support this control.
     */
    public LDAPPagedResultsControl(int pageSize, boolean critical)
    {
        this(pageSize, null, critical);
        return;
    }

    /**
     * Returns the current page size value
     */
    public int getPageSize()
    {
        return _pageSize;
    }

    /**
     * Sets the request page size value
     *
     * @param pageSize new page size value
     * 
     */
    public void setPageSize(int pageSize)
    {
        _pageSize = pageSize;
        setEncodedValue();
    }

    /**
     * Returns the current cookie
     */
    public byte[] getCookie()
    {
        return _cookie;
    }

    /**
     * Sets new cookie value
     *
     * @param cookie new cookie value
     *
     */
    public void setCookie(byte cookie[]) {
        _cookie = cookie;
        setEncodedValue();
    }

}
